#ifndef ISUBSYS_H
#define ISUBSYS_H

#include "pstring.h"

class PBotSerialParser;

/*
    This class is an interface for all serial based SubSystems.  This interface allows the serial command parser to
    message the SubSystem when an incomming command is intended for that SubSytem.  This class should NEVER be directly
    instantiaded.  The micro-controller environment precludes the use of pure virtual functions but this class is an
    abstract interface only.
*/
class ISubSys
{
public:
    virtual ~ISubSys() {};  // Required virtual destructor

    // Optional overrides
    virtual int isReady() { return ready; };                    // Returns the ready status variable
    virtual int attachParser(PBotSerialParser* parser);         // Attach the serial command parser to the subsystem
    virtual const String& getResult() { return strReturn; };    // Called to get string representing the results of the requested command.

    // Required overrides
    virtual int execCmd() {return -1;};                         // Called when a command string has been captured for this subsystem.


protected:
    PBotSerialParser*   p_parser;                               // Parser object to use to get commands and return responses
    String              strReturn;                              // String for constructiong return values
    int                 ready;                                  // Indicates if the object is ready to process
};


/** @brief Attach a command parser to this subsystem to use for communication to the host.
 
    @return int: 0 for success, -1 for failure.
 */
inline int ISubSys::attachParser(PBotSerialParser* parser)
{
    if (parser)
    {
        p_parser = parser;
        return 0;
    }

    return -1;
}

#endif // ISUBSYS_H
