#ifndef SONARSENSOR_H
#define SONARSENSOR_H

#include "isonarsensor.h"

/*
    This class represents any of a number of basic one or two pin sonar sensors that measure
    distance using an output trigger pulse and then listen for an echo that is timed using an input
    pulse.  This object can handle either type.  It will only operate sensors that use the common
    positive pulse logic.  Other types of sensors or those requiring more interaction should be
    implemented in a more specific class.
*/

class SonarSensor : public ISonarSensor
{
public:
    SonarSensor();                  
    SonarSensor(int t_pin, int e_pin, int t_len, int t_out);
    void init(int t_pin, int e_pin, int t_len, int t_out);

    // Interface overrides: See interface defintions for documentation
    int trigger();
    int getRange_in();
    int getRange_cm();

protected:
    int trigPin;        // Trigger pin to start ranging cycle
    int echoPin;        // Echo pin to capture time-of-flight
    int trigLen;        // Length in usecs for the trigger pulse duration
    int echoTimeout;    // Maximum expected echo length
    int tof;            // Last time of flight in usecs
};

#endif // SONARSENSOR_H
