#ifndef SONARSUBSYS_H
#define SONARSUBSYS_H

#include "isubsys.h"
#include "pbotserialparser.h"

#define SN_ACT_RD           "R"
#define SN_ACT_WR           "W"
#define SN_ALL_SENSORS      "A"
#define SN_TARG_SENSOR      "S"
#define SN_TARG_SCAN_RATE   "RT"
#define SN_TARG_MODE        "M"
#define SN_TARG_UNITS       "U"
#define SN_TARG_SCANPAT     "SP"

enum SNSUB_PARAMS
{
    SN_MAX_SENSORS          = 8,
    SN_MODE_SINGLE_SWEEP    = 0,
    SN_MODE_CONT_SWEEP      = 1,
    SN_IN                   = 0,
    SN_CM                   = 1,
};

class ISonarSensor;

struct SubSysData
{
    int mode;
    int rate;
    int cnt;
    ISonarSensor* p_sensorArry[SN_MAX_SENSORS];
};

/**
    This class implements the ISubSys interface to create a sonar sensor sub-system that can manage 8 sonar sensors
    controlled via a command interface from another host controller.  The subsystem processes commands that are 
    sent to it as string data.  All results are returned to the command interface as string data as well for 
    transmission to the controlling host.
 */
class SonarSubSys : public ISubSys
{
public:
    SonarSubSys();
    SonarSubSys(int r, int m, int u=SN_IN);
    SonarSubSys(PBotSerialParser& parser, int r, int m, int u=SN_IN);
    ~SonarSubSys() {};
    void init(int r, int m, int u=SN_IN);
    void init(PBotSerialParser& parser, int r, int m, int u=SN_IN);

    // Overrides
    int isReady();                              // Indicates if the SubSystem is initialized and ready to operate
    int execCmd();                              // Called to when there is a command that needs to be processed

    // Command Handeling functions
    int readSensor(int i);                      
    int readAllSensors();                       
    int readScanRate();                         
    int readMode();                             
    int writeScanRate(const String& term);      
    int writeScanPattern(const String& term);   
    int writeMode(const String& term);
    int writeUnits(const String& term);

    // Custom public interface
    int addSensor(ISonarSensor& sensor);        // Insert a sonar sensor object into the internal array
    int removeSensor(int i);                    // Remove the sonar sensor from the array at index i

protected:
    int clrArrays();                            // Clear the data in the arrays and reset state tracking
    int setRate(int r);
    int setMode(int m);
    int setUnits(int u);

protected:
    int         units;                          // The units for all range readings
    volatile    SubSysData mbox;                // Shared data structure for the cog running the scanning task.
};


#endif // SONARSUBSYS_H
